@@ -7,7 +7,7 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-__version__ = '2.2.0'
+__version__ = '2.2.1'
 __author__ = "Davide Brunato"
 __contact__ = "brunato@sissa.it"
 __copyright__ = "Copyright 2018-2021, SISSA"
@@ -23,7 +23,7 @@ from .exceptions import ElementPathError, MissingContextError, \
 from .xpath_context import XPathContext, XPathSchemaContext
 from .xpath_nodes import XPathNode, AttributeNode, TextNode, \
     NamespaceNode, TypedElement, TypedAttribute
-from .xpath_token import XPathToken
+from .xpath_token import XPathToken, XPathFunction
 from .xpath1 import XPath1Parser
 from .xpath2 import XPath2Parser
 from .xpath_selectors import select, iter_select, Selector
@@ -36,5 +36,6 @@ __all__ = ['ElementPathError', 'MissingContextError', 'ElementPathSyntaxError',
            'ElementPathOverflowError', 'ElementPathValueError', 'ElementPathTypeError',
            'ElementPathZeroDivisionError', 'datatypes', 'XPathContext', 'XPathSchemaContext',
            'XPathNode', 'AttributeNode', 'TextNode', 'NamespaceNode', 'TypedAttribute',
-           'TypedElement', 'XPathToken', 'XPath1Parser', 'XPath2Parser', 'select',
-           'iter_select', 'Selector', 'AbstractSchemaProxy', 'RegexError', 'translate_pattern']
+           'TypedElement', 'XPathToken', 'XPathFunction', 'XPath1Parser', 'XPath2Parser',
+           'select', 'iter_select', 'Selector', 'AbstractSchemaProxy',
+           'RegexError', 'translate_pattern']
