@@ -200,7 +200,7 @@ class XPathToken(Token):
                 msg = "missing %s argument" % ordinal(index + 1)
                 raise self.error('XPST0017', msg) from None
             else:
-                return
+                return default
         else:
             item = None
             for k, result in enumerate(selector(copy(context))):
@@ -226,7 +226,7 @@ class XPathToken(Token):
                     raise self.error('XPTY0004', msg.format(ord_arg))
 
         # Type promotion checking (see "function conversion rules" in XPath 2.0 language definition)
-        if cls is not None and not isinstance(item, cls):
+        if cls is not None and not isinstance(item, cls) and not issubclass(cls, XPathToken):
             if promote and isinstance(item, promote):
                 return cls(item)
 
@@ -242,12 +242,11 @@ class XPathToken(Token):
                 value = self.data_value(item)
                 if isinstance(value, cls):
                     return value
+                elif isinstance(value, AnyURI) and issubclass(cls, str):
+                    return cls(value)
                 elif isinstance(value, UntypedAtomic):
                     try:
-                        if issubclass(cls, str):
-                            return str(value)
-                        else:
-                            return cls(value)
+                        return cls(value)
                     except (TypeError, ValueError):
                         pass
 
@@ -378,7 +377,8 @@ class XPathToken(Token):
 
         :param context: the XPath dynamic context.
         """
-        self.parser.check_variables(context.variables)
+        if context is not None:
+            self.parser.check_variables(context.variables)
 
         for result in self.select(context):
             if not isinstance(result, XPathNode):
@@ -393,7 +393,10 @@ class XPathToken(Token):
                 else:
                     yield result.value
             elif isinstance(result, NamespaceNode):  # pragma: no cover
-                yield result.prefix, result.uri
+                if self.parser.compatibility_mode:
+                    yield result.prefix, result.uri
+                else:
+                    yield result.uri
 
     def get_results(self, context):
         """
@@ -468,13 +471,15 @@ class XPathToken(Token):
 
         return op1, op2
 
-    def get_absolute_uri(self, uri, base_uri=None):
+    def get_absolute_uri(self, uri, base_uri=None, as_string=True):
         """
         Obtains an absolute URI from the argument and the static context.
 
         :param uri: a string representing an URI.
         :param base_uri: an alternative base URI, otherwise the base_uri \
         of the static context is used.
+        :param as_string: if `True` then returns the URI as a string, otherwise \
+        returns the URI as xs:anyURI instance.
         :returns: the argument if it's an absolute URI. Otherwise returns the URI
         obtained by the join o the base_uri of the static context with the
         argument. Returns the argument if the base_uri is `None'.
@@ -486,14 +491,16 @@ class XPathToken(Token):
         if url_parts.scheme or url_parts.netloc \
                 or url_parts.path.startswith('/') \
                 or base_uri is None:
-            return uri
+            return uri if as_string else AnyURI(uri)
 
         url_parts = urllib.parse.urlsplit(base_uri)
         if url_parts.fragment or not url_parts.scheme and \
                 not url_parts.netloc and not url_parts.path.startswith('/'):
             raise self.error('FORG0002', '{!r} is not suitable as base URI'.format(base_uri))
 
-        return urllib.parse.urljoin(base_uri, uri)
+        if as_string:
+            return urllib.parse.urljoin(base_uri, uri)
+        return AnyURI(urllib.parse.urljoin(base_uri, uri))
 
     def get_namespace(self, prefix):
         """
@@ -621,7 +628,7 @@ class XPathToken(Token):
                     if xsd_node.value.is_matching(name):
                         if xsd_node.name is None:
                             # node is an XSD attribute wildcard
-                            xsd_node = self.parser.schema.get_attribute(name)
+                            xsd_node = schema_context.root.maps.attributes.get(name)
                             if xsd_node is None:
                                 continue
 
@@ -632,7 +639,7 @@ class XPathToken(Token):
                 elif xsd_node.is_matching(name, self.parser.default_namespace):
                     if xsd_node.name is None:
                         # node is an XSD element wildcard
-                        xsd_node = self.parser.schema.get_element(name)
+                        xsd_node = schema_context.root.maps.elements.get(name)
                         if xsd_node is None:
                             continue
 
@@ -1081,18 +1088,23 @@ class XPathFunction(XPathToken):
     """
     A token for processing XPath functions.
     """
-    pattern = r'\b[^\d\W][\w.\-\xb7\u0300-\u036F\u203F\u2040]*(?=\s*(?:\(\:.*\:\))?\s*\((?!\:))'
-    nargs = None
     _name = None
+    pattern = r'\b[^\d\W][\w.\-\xb7\u0300-\u036F\u203F\u2040]*(?=\s*(?:\(\:.*\:\))?\s*\((?!\:))'
+
+    sequence_types = ()
+    "Sequence types of arguments and of the return value of the function."
+
+    nargs = None
+    "Number of arguments: a single value or a couple with None that means unbounded."
 
     def __init__(self, parser, nargs=None):
         super().__init__(parser)
         if isinstance(nargs, int) and nargs != self.nargs:
             if nargs < 0:
-                raise ElementPathValueError('number of arguments must be non negative')
+                raise self.error('XPST0017', 'number of arguments must be non negative')
             elif isinstance(self.nargs, int) or isinstance(self.nargs, (tuple, list)) and \
                     (self.nargs[0] > nargs or self.nargs[1] and self.nargs[1] < nargs):
-                raise ElementPathValueError('incongruent number of arguments')
+                raise self.error('XPST0017', 'incongruent number of arguments')
             else:
                 self.nargs = nargs
 
@@ -1102,16 +1114,22 @@ class XPathFunction(XPathToken):
             for token in argument_list:
                 args.append(token)
         elif isinstance(argument_list, XPathToken):
-            for token in argument_list.iter():
-                if token.symbol not in ('(', ','):
-                    args.append(token)
+            if argument_list.symbol == '(':
+                args.append(argument_list)
+            else:
+                for token in argument_list.iter():
+                    if token.symbol not in ('(', ','):
+                        args.append(token)
 
         context = copy(context)
         if self.symbol == 'function':
             if context is None:
                 raise self.missing_context()
 
-            for variable, value in zip(self, args):
+            for variable, sequence_type, value in zip(self, self.sequence_types, args):
+                if not self.parser.match_sequence_type(value, sequence_type):
+                    msg = "invalid type for argument {!r}"
+                    raise self.error('XPTY0004', msg.format(variable[0].value))
                 context.variables[variable[0].value] = value
         elif any(tk.symbol == '?' for tk in self):
             for value, tk in zip(args, filter(lambda x: x.symbol == '?', self)):
@@ -1124,7 +1142,12 @@ class XPathFunction(XPathToken):
                 else:
                     self.append(ValueToken(self.parser, value=value))
 
-        return self.evaluate(context)
+        result = self.evaluate(context)
+        if not self.parser.match_sequence_type(result, self.sequence_types[-1]):
+            msg = "{!r} does not match sequence type {}"
+            raise self.error('XPTY0004', msg.format(result, self.sequence_types[-1]))
+
+        return result
 
     @property
     def name(self):
